"""Adobe Customer Journey Analytics MCP Server.

A Model Context Protocol server for Adobe Customer Journey Analytics (CJA)
that enables LLMs to perform advanced data analytics queries.
"""

__version__ = "0.1.0"
